//Directives préprocesseur: chargement des bibliothèques SDL2 et Glad
#include <SDL2/SDL.h>
#include "glad/glad.h"
	
//Fonction de rendu
void render(SDL_Window* win){
	
	// Choisi une couleur de fond (au format rouge, vert, bleu et alpha)
	glClearColor(0.0, 0.0, 0.0, 1.0);
	// Applique la couleur de fond
	glClear(GL_COLOR_BUFFER_BIT);
	
	/* Met à jour le résultat*/
	SDL_GL_SwapWindow(win);
}


//Fonction qui libère les ressources
void freeRessources(SDL_Window*win,SDL_GLContext gContext){

    //Supprime le contexte OpenGL
    SDL_GL_DeleteContext(gContext);
    //Détruit la fenêtre
    SDL_DestroyWindow(win);
    //Quitte la SDL2
    SDL_Quit();

}	
		
void mainLoop(SDL_Window *win){
    //Boolean qui permet de quitter la boucle
    SDL_bool quit=SDL_FALSE;
    //Evênement SDL (e)
    SDL_Event e;
    //Boucle principale (Boucle si la variable quit = faux)
    while(!quit){
        //Test si un événement à lieu
        if(SDL_PollEvent(&e)!=0){
            /*Si c'est un événement de type SDL_QUIT Fermeture de la fenêtre
            la variable "quit" aura la valeur true.=> 1 et on sortira de la boucle.*/
            if(e.type==SDL_QUIT)
                quit=SDL_TRUE;
            //Teste si une touche est pressée.
            else if(e.type==SDL_KEYDOWN){
				//Si on appuie sur la touche ESCAPE => On quitte la boucle (quit=true).
                if(e.key.keysym.sym==SDLK_ESCAPE)
                    quit=SDL_TRUE;
            }
        }
		//Appel la fonction de rendu
		render(win);
    }
}



//Fonction principale, point d'entrée du programme
int main(int argc, char* argv[]){
	
    //Fonction d'initialisation de la SDL (seulement la vidéo dans ce tutoriel)
    SDL_Init(SDL_INIT_VIDEO);
    
    //Définition des attributs du contexte OpenGL
    //Active le double buffering
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    //Version d'OpenGL que nous souhaitons utiliser => 2.1
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 1);
    //Cette fonction force l'utilisation des fonctions dites CORE d'OpenGL 2.1
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);

    
    //Crée la fenêtre SDL (titre, position, taille, utilisable avec un contexte OpenGL)
    SDL_Window* win =SDL_CreateWindow("Triangle SDL2 + OpenGL",
        SDL_WINDOWPOS_CENTERED,SDL_WINDOWPOS_CENTERED,
        640,480,
        SDL_WINDOW_OPENGL|SDL_WINDOW_OPENGL);

    //Crée le contexte OpenGL => gContext
    SDL_GLContext gContext=SDL_GL_CreateContext(win);
    
        //Chargement de Glad
    int version = gladLoadGLLoader((GLADloadproc) SDL_GL_GetProcAddress);
    if(!version){
        printf("Glad was not initialized.\n");
        exit(1);
    }
    else{
        printf("OpenGL Renderer: %s\n",glGetString(GL_VENDOR));
        printf("OpenGL Renderer: %s\n",glGetString(GL_VERSION));
    }
    
	mainLoop(win);
	//Libère les ressources du programme.
    freeRessources(win,gContext);

	//Fin réussie du programme, retourne 0
	return EXIT_SUCCESS;
}
